module iic(clk, scl, sda, ccp, reset, mclr_n, led, sled, txd);
	input clk, scl, sda, ccp, reset;
	output mclr_n, txd;
	output [7:0] led;
	output [11:0] sled;
	assign led[0] = scl;
	assign led[1] = sda;
	assign led[2] = ccp;
	assign led[6:3] = 4'b0000;
	assign led[7] = mclr_n;
	reg [15:0] count = 0;
	always @(posedge clk) begin
		if (reset) count <= 0;
		else if (count != 16'hffff) count <= count + 1;
	end
	assign mclr_n = count == 16'hffff;
	assign sled = 12'hfff;
	reg mclr_n1 = 0;
	always @(posedge clk) mclr_n1 <= mclr_n;
	analizer analizer(.clk(clk), .start(mclr_n & ~mclr_n1), .data({ sda, scl }), .txd(txd));
endmodule

module uart(clk, data, start, txd);
	input clk, start;
	input [7:0] data;
	output txd;
	parameter START_POS = 15;
	parameter STOP_POS = 9;
	parameter TIMING = 433; // 115200baud@50MHz
	reg [3:0] bitcount = STOP_POS;
	reg [8:0] timing = 0;
	always @(posedge clk) begin
		if (bitcount == STOP_POS & start) begin
			bitcount <= START_POS;
			timing <= 0;
		end
		else if (bitcount != STOP_POS & timing == TIMING) begin
			bitcount <= bitcount + 1;
			timing <= 0;
		end
		else timing <= timing + 1;
	end
	assign txd = bitcount == START_POS ? 0 : bitcount[3] ? 1 : data[bitcount];
endmodule

module analizer(clk, start, data, txd);
	input clk, start;
	input [1:0] data;
	output txd;
	parameter TRIGGER_WAIT = 1;
	parameter SAMPLE_WAIT = 7;
	reg [1:0] state = 0;
	reg [12:0] adr = 0;
	reg [13:0] timing = 0;
	reg [1:0] mem[0:8191];
	reg [1:0] tdata = 0;
	reg [6:0] column = 0;
	always @(posedge clk) begin
		if (state == 0) begin
			if (start) state <= 1;
			adr <= 0;
		end
		else if (state == 1) begin
			timing <= timing + 1;
			if (timing == TRIGGER_WAIT) begin
				timing <= 0;
				state <= 2;
			end
		end
		else if (state == 2) begin
			timing <= timing + 1;
			if (timing == SAMPLE_WAIT) begin
				timing <= 0;
				if (adr == 13'h1fff) state <= 3;
				adr <= adr + 1;
			end
		end
		else if (state == 3) begin
			timing <= timing + 1;
			if (timing == 14'h3fff) begin
				if (column != 63 & column != 64) adr <= adr + 1;
				if (column == 65) begin
					column <= 0;
					if (adr == 13'h1fff) state <= 0;
				end
				else column <= column + 1;
			end
		end
	end
	always @(posedge clk) begin
		if (state == 2) mem[adr] <= data;
		tdata <= mem[adr];
	end
	function [7:0] d;
		input [6:0] column;
		input [1:0] tdata;
		case (column)
			64: d = 13;
			65: d = 10;
			default d = { 6'b001100, tdata };
		endcase
	endfunction
	uart uart(.clk(clk), .start(state == 3 & timing == 0), .data(d(column, tdata)), .txd(txd));
endmodule

